@@ -3,7 +3,7 @@
 import logging
 import os
 
-from rasterio.path import parse_path, UnparsedPath
+from rasterio._path import _parse_path, _UnparsedPath
 
 
 log = logging.getLogger(__name__)
@@ -15,7 +15,7 @@ except ImportError:
     boto3 = None
 
 
-class Session(object):
+class Session:
     """Base for classes that configure access to secured resources.
 
     Attributes
@@ -95,9 +95,9 @@ class Session(object):
         if not path:
             return DummySession
 
-        path = parse_path(path)
+        path = _parse_path(path)
 
-        if isinstance(path, UnparsedPath) or path.is_local:
+        if isinstance(path, _UnparsedPath) or path.is_local:
             return DummySession
 
         elif (
