@@ -21,22 +21,24 @@ import collections
 from collections.abc import Iterable
 import functools
 import math
+import warnings
 
 from affine import Affine
 import attr
 import numpy as np
 
-from rasterio.errors import WindowError
+from rasterio.errors import WindowError, RasterioDeprecationWarning
 from rasterio.transform import rowcol, guard_transform
 
 
-class WindowMethodsMixin(object):
+class WindowMethodsMixin:
     """Mixin providing methods for window-related calculations.
     These methods are wrappers for the functionality in
     `rasterio.windows` module.
 
     A subclass with this mixin MUST provide the following
-    properties: `transform`, `height` and `width`
+    properties: `transform`, `height` and `width`.
+
     """
 
     def window(self, left, bottom, right, top, precision=None):
@@ -56,17 +58,27 @@ class WindowMethodsMixin(object):
         top: float
             Top (north) bounding coordinate
         precision: int, optional
-            Number of decimal points of precision when computing inverse
-            transform.
+            This parameter is unused, deprecated in rasterio 1.3.0, and
+            will be removed in version 2.0.0.
 
         Returns
         -------
         window: Window
+
         """
-        transform = guard_transform(self.transform)
+        if precision is not None:
+            warnings.warn(
+                "The precision parameter is unused, deprecated, and will be removed in 2.0.0.",
+                RasterioDeprecationWarning,
+            )
+
         return from_bounds(
-            left, bottom, right, top, transform=transform,
-            height=self.height, width=self.width, precision=precision)
+            left,
+            bottom,
+            right,
+            top,
+            transform=guard_transform(self.transform),
+        )
 
     def window_transform(self, window):
         """Get the affine transform for a dataset window.
@@ -80,8 +92,8 @@ class WindowMethodsMixin(object):
         -------
         transform: Affine
             The affine transform matrix for the given window
-        """
 
+        """
         gtransform = guard_transform(self.transform)
         return transform(window, gtransform)
 
@@ -97,15 +109,15 @@ class WindowMethodsMixin(object):
         -------
         bounds : tuple
             x_min, y_min, x_max, y_max for the given window
-        """
 
+        """
         transform = guard_transform(self.transform)
         return bounds(window, transform)
 
 
 def iter_args(function):
-    """Decorator to allow function to take either *args or
-    a single iterable which gets expanded to *args.
+    """Decorator to allow function to take either ``*args`` or
+    a single iterable which gets expanded to ``*args``.
     """
     @functools.wraps(function)
     def wrapper(*args, **kwargs):
@@ -140,36 +152,53 @@ def get_data_window(arr, nodata=None):
     -------
     Window
     """
-
-    num_dims = len(arr.shape)
-    if num_dims > 3:
+    if not 0 < arr.ndim <=3 :
         raise WindowError(
-            "get_data_window input array must have no more than "
-            "3 dimensions")
+            "get_data_window input array must have 1, 2, or 3 dimensions")
 
-    if nodata is None:
-        if not hasattr(arr, 'mask'):
-            return Window.from_slices((0, arr.shape[-2]), (0, arr.shape[-1]))
+    # If nodata is defined, construct mask from that value
+    # Otherwise retrieve mask from array (if it is masked)
+    # Finally try returning a full window (nodata=None and nothing in arr is masked)
+    if nodata is not None:
+        arr_mask = arr != nodata
+    elif np.ma.is_masked(arr):
+        arr_mask = ~np.ma.getmask(arr)
     else:
-        arr = np.ma.masked_array(arr, arr == nodata)
+        if arr.ndim == 1:
+            full_window = ((0, arr.size), (0, 0))
+        else:
+            full_window = ((0, arr.shape[-2]), (0, arr.shape[-1]))
+        return Window.from_slices(*full_window)
 
-    if num_dims == 2:
-        data_rows, data_cols = np.where(np.equal(arr.mask, False))
-    else:
-        data_rows, data_cols = np.where(
-            np.any(np.equal(np.rollaxis(arr.mask, 0, 3), False), axis=2))
+    if arr.ndim == 3:
+        arr_mask = np.any(arr_mask, axis=0)
 
-    if data_rows.size:
-        row_range = (data_rows.min(), data_rows.max() + 1)
-    else:
-        row_range = (0, 0)
+    # We only have 1 or 2 dimension cases to process
+    v = []
+    for nz in arr_mask.nonzero():
+        if nz.size:
+            v.append((nz.min(), nz.max() + 1))
+        else:
+            v.append((0, 0))
+    
+    if arr_mask.ndim == 1:
+        v.append((0, 0))
+        
+    return Window.from_slices(*v)
 
-    if data_cols.size:
-        col_range = (data_cols.min(), data_cols.max() + 1)
-    else:
-        col_range = (0, 0)
 
-    return Window.from_slices(row_range, col_range)
+def _compute_union(w1, w2):
+    """Compute the union of two windows"""
+    col_off = min(w1.col_off, w2.col_off)
+    row_off = min(w1.row_off, w2.row_off)
+    width = max(w1.col_off+w1.width, w2.col_off+w2.width) - col_off
+    height = max(w1.row_off+w1.height, w2.row_off+w2.height) - row_off
+    return col_off, row_off, width, height
+
+
+def _union(w1, w2):
+    coeffs = _compute_union(w1, w2)
+    return Window(*coeffs)
 
 
 @iter_args
@@ -186,15 +215,7 @@ def union(*windows):
     -------
     Window
     """
-    stacked = np.dstack([toranges(w) for w in windows])
-    row_start, row_stop = stacked[0, 0].min(), stacked[0, 1].max()
-    col_start, col_stop = stacked[1, 0].min(), stacked[1, 1].max()
-    return Window(
-        col_off=col_start,
-        row_off=row_start,
-        width=col_stop - col_start,
-        height=row_stop - row_start,
-    )
+    return functools.reduce(_union, windows)
 
 
 @iter_args
@@ -212,18 +233,25 @@ def intersection(*windows):
     -------
     Window
     """
-    if not intersect(windows):
-        raise WindowError("windows do not intersect")
+    return functools.reduce(_intersection, windows)
 
-    stacked = np.dstack([toranges(w) for w in windows])
-    row_start, row_stop = stacked[0, 0].max(), stacked[0, 1].min()
-    col_start, col_stop = stacked[1, 0].max(), stacked[1, 1].min()
-    return Window(
-        col_off=col_start,
-        row_off=row_start,
-        width=col_stop - col_start,
-        height=row_stop - row_start,
-    )
+
+def _compute_intersection(w1, w2):
+    """ Compute intersection of window 1 and window 2"""
+    col_off = max(w1.col_off, w2.col_off)
+    row_off = max(w1.row_off, w2.row_off)
+    width = min(w1.col_off+w1.width, w2.col_off+w2.width) - col_off
+    height = min(w1.row_off+w1.height, w2.row_off+w2.height) - row_off
+    return col_off, row_off, width, height
+
+
+def _intersection(w1, w2):
+    """ Compute intersection of window 1 and window 2"""
+    coeffs = _compute_intersection(w1, w2)
+    if coeffs[2] > 0 and coeffs[3] > 0:
+        return Window(*coeffs)
+    else:
+        raise WindowError(f"Intersection is empty {w1} {w2}")
 
 
 @iter_args
@@ -240,20 +268,11 @@ def intersect(*windows):
     bool
         True if all windows intersect.
     """
-    from itertools import combinations
-
-    def intersects(range1, range2):
-        return not (
-            range1[0] >= range2[1] or range1[1] <= range2[0])
-
-    windows = np.array([toranges(w) for w in windows])
-
-    for i in (0, 1):
-        for c in combinations(windows[:, i], 2):
-            if not intersects(*c):
-                return False
-
-    return True
+    try:
+        intersection(*windows)
+        return True
+    except WindowError:
+        return False
 
 
 def from_bounds(
@@ -273,13 +292,9 @@ def from_bounds(
         Top (north) bounding coordinates
     transform: Affine, required
         Affine transform matrix.
-    height: int, required
-        Number of rows of the window.
-    width: int, required
-        Number of columns of the window.
-    precision: int or float, optional
-        An integer number of decimal points of precision when computing
-        inverse transform, or an absolute float precision.
+    precision, height, width: int, optional
+        These parameters are unused, deprecated in rasterio 1.3.0, and
+        will be removed in version 2.0.0.
 
     Returns
     -------
@@ -292,12 +307,16 @@ def from_bounds(
         If a window can't be calculated.
 
     """
+    if height is not None or width is not None or precision is not None:
+        warnings.warn(
+            "The height, width, and precision parameters are unused, deprecated, and will be removed in 2.0.0.",
+            RasterioDeprecationWarning,
+        )
+
     if not isinstance(transform, Affine):  # TODO: RPCs?
         raise WindowError("A transform object is required to calculate the window")
-
     if (right - left) / transform.a < 0:
         raise WindowError("Bounds and transform are inconsistent")
-
     if (bottom - top) / transform.e < 0:
         raise WindowError("Bounds and transform are inconsistent")
 
@@ -306,7 +325,6 @@ def from_bounds(
         [left, right, right, left],
         [top, top, bottom, bottom],
         op=float,
-        precision=precision,
     )
     row_start, row_stop = min(rows), max(rows)
     col_start, col_stop = min(cols), max(cols)
@@ -333,9 +351,9 @@ def transform(window, transform):
     -------
     Affine
         The affine transform matrix for the given window
+
     """
     window = evaluate(window, height=0, width=0)
-
     x, y = transform * (window.col_off or 0.0, window.row_off or 0.0)
     return Affine.translation(
         x - transform.c, y - transform.f) * transform
@@ -458,13 +476,10 @@ def window_index(window, height=0, width=0):
     -------
     row_slice, col_slice: slice
         A pair of slices in row, column order
+
     """
     window = evaluate(window, height=height, width=width)
-
-    (row_start, row_stop), (col_start, col_stop) = window.toranges()
-    return (
-        slice(int(math.floor(row_start)), int(math.ceil(row_stop))),
-        slice(int(math.floor(col_start)), int(math.ceil(col_stop))))
+    return window.toslices()
 
 
 def round_window_to_full_blocks(window, block_shapes, height=0, width=0):
@@ -512,7 +527,7 @@ def validate_length_value(instance, attribute, value):
 
 @attr.s(slots=True,
         frozen=True)
-class Window(object):
+class Window:
     """Windows are rectangular subsets of rasters.
 
     This class abstracts the 2-tuples mentioned in the module docstring
@@ -525,6 +540,8 @@ class Window(object):
     width, height: float
         Lengths of the window.
 
+    Notes
+    -----
     Previously the lengths were called 'num_cols' and 'num_rows' but
     this is a bit confusing in the new float precision world and the
     attributes have been changed. The originals are deprecated.
@@ -575,8 +592,23 @@ class Window(object):
         -------
         row_slice, col_slice: slice
             A pair of slices in row, column order
+
         """
-        return tuple(slice(*rng) for rng in self.toranges())
+        (r0, r1), (c0, c1) = self.toranges()
+
+        if r0 < 0:
+            r0 = 0
+        if r1 < 0:
+            r1 = 0
+        if c0 < 0:
+            c0 = 0
+        if c1 < 0:
+            c1 = 0
+
+        return (
+            slice(int(math.floor(r0)), int(math.ceil(r1))),
+            slice(int(math.floor(c0)), int(math.ceil(c1))),
+        )
 
     @classmethod
     def from_slices(cls, rows, cols, height=-1, width=-1, boundless=False):
@@ -671,7 +703,7 @@ class Window(object):
         return cls(col_off=col_off, row_off=row_off, width=num_cols,
                    height=num_rows)
 
-    def round_lengths(self, op='floor', pixel_precision=None):
+    def round_lengths(self, **kwds):
         """Return a copy with width and height rounded.
 
         Lengths are rounded to the preceding (floor) or succeeding (ceil)
@@ -679,31 +711,27 @@ class Window(object):
 
         Parameters
         ----------
-        op: str
-            'ceil' or 'floor'
-        pixel_precision: int, optional (default: None)
-            Number of places of rounding precision.
+        kwds : dict
+            Collects keyword arguments that are no longer used.
 
         Returns
         -------
         Window
+
         """
-        if op not in {'ceil', 'floor'}:
-            raise WindowError("operator must be 'ceil' or 'floor', got '{}'".format(op))
+        operator = lambda x: int(math.floor(x + 0.5))
+        width = operator(self.width)
+        height = operator(self.height)
+        return Window(self.col_off, self.row_off, width, height)
 
-        operator = getattr(math, op)
-        if pixel_precision is None:
-            return Window(self.col_off, self.row_off,
-                          operator(self.width), operator(self.height))
-        else:
-            return Window(self.col_off, self.row_off,
-                          operator(round(self.width, pixel_precision)),
-                          operator(round(self.height, pixel_precision)))
+    def round_shape(self, **kwds):
+        warnings.warn(
+            "round_shape is deprecated and will be removed in Rasterio 2.0.0.",
+            RasterioDeprecationWarning,
+        )
+        return self.round_lengths(**kwds)
 
-    # TODO: deprecate round_shape at 1.3.0, with a warning.
-    round_shape = round_lengths
-
-    def round_offsets(self, op='floor', pixel_precision=None):
+    def round_offsets(self, **kwds):
         """Return a copy with column and row offsets rounded.
 
         Offsets are rounded to the preceding (floor) or succeeding (ceil)
@@ -711,26 +739,18 @@ class Window(object):
 
         Parameters
         ----------
-        op : str
-            'ceil' or 'floor'
-        pixel_precision : int, optional (default: None)
-            Number of places of rounding precision.
+        kwds : dict
+            Collects keyword arguments that are no longer used.
 
         Returns
         -------
         Window
-        """
-        if op not in {'ceil', 'floor'}:
-            raise WindowError("operator must be 'ceil' or 'floor', got '{}'".format(op))
 
-        operator = getattr(math, op)
-        if pixel_precision is None:
-            return Window(operator(self.col_off), operator(self.row_off),
-                          self.width, self.height)
-        else:
-            return Window(operator(round(self.col_off, pixel_precision)),
-                          operator(round(self.row_off, pixel_precision)),
-                          self.width, self.height)
+        """
+        operator = lambda x: int(math.floor(x + 0.001))
+        row_off = operator(self.row_off)
+        col_off = operator(self.col_off)
+        return Window(col_off, row_off, self.width, self.height)
 
     def crop(self, height, width):
         """Return a copy cropped to height and width"""
